from unittest import mock

from tests.testcase import HookeeTestCase

__author__ = "Alex Laird"
__copyright__ = "Copyright 2020, Alex Laird"
__version__ = "2.0.0"


class TestConfig(HookeeTestCase):
    def test_write_updates_file(self):
        # GIVEN
        with open(self.config.config_path) as f:
            contents = f.read()
        self.assertNotIn("test123", contents)

        # WHEN
        self.config._update_config_objects("auth_token", "test123")
        with open(self.config.config_path) as f:
            updated_contents = f.read()

        # THEN
        self.assertNotEqual(contents, updated_contents)
        self.assertIn("test123", updated_contents)

    @mock.patch("hookee.conf.Config._write_config_objects_to_file")
    def test_config_set(self, mock_write_config_objects_to_file):
        # WHEN
        self.config.set("auth_token", "test123")

        # THEN
        self.assertEqual(self.config.get("auth_token"), "test123")
        self.assertTrue(mock_write_config_objects_to_file.called)

    @mock.patch("hookee.conf.Config._write_config_objects_to_file")
    def test_config_append(self, mock_write_config_objects_to_file):
        # GIVEN
        self.assertNotIn("foo_bar", self.config.get("plugins"))

        # WHEN
        self.config.append("plugins", "foo_bar")

        # THEN
        self.assertIn("foo_bar", self.config.get("plugins"))
        self.assertTrue(mock_write_config_objects_to_file.called)

    @mock.patch("hookee.conf.Config._write_config_objects_to_file")
    def test_config_remove(self, mock_write_config_objects_to_file):
        # GIVEN
        self.assertIn("request_headers", self.config.get("plugins"))

        # WHEN
        self.config.remove("plugins", "request_headers")

        # THEN
        self.assertNotIn("request_headers", self.config.get("plugins"))
        self.assertTrue(mock_write_config_objects_to_file.called)



import os
import shutil
import unittest

from click.testing import CliRunner

from hookee.conf import Config

__author__ = "Alex Laird"
__copyright__ = "Copyright 2020, Alex Laird"
__version__ = "1.2.4"


class HookeeTestCase(unittest.TestCase):
    def setUp(self):
        self.config_dir = os.path.normpath(
            os.path.join(os.path.abspath(os.path.dirname(__file__)), ".config", "hookee"))
        if not os.path.exists(self.config_dir):
            os.makedirs(self.config_dir)

        os.environ["HOOKEEDIR"] = self.config_dir

        self.config = Config()

        self.plugins_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), "plugins_dir")
        self.config.set("plugins_dir", self.plugins_dir)

        self.runner = CliRunner()

    def tearDown(self):
        if os.path.exists(self.config_dir):
            shutil.rmtree(self.config_dir)



import time

import requests
from hookee.conf import Config

from hookee import HookeeManager
from tests.testcase import HookeeTestCase

__author__ = "Alex Laird"
__copyright__ = "Copyright 2020, Alex Laird"
__version__ = "1.2.0"


class TestHookeeManagerEdges(HookeeTestCase):
    def test_not_click_logging(self):
        self.assertFalse(self.config.click_logging)
    
    def test_hookee_manager(self):
        # GIVEN
        hookee_manager = HookeeManager()
        hookee_manager._init_server_and_tunnel()

        self.assertIsNotNone(hookee_manager.server._thread)
        self.assertIsNotNone(hookee_manager.tunnel._thread)

        # WHEN
        hookee_manager.stop()

        # Wait for things to tear down
        time.sleep(2)

        # THEN
        self.assertIsNone(hookee_manager.server._thread)
        self.assertIsNone(hookee_manager.tunnel._thread)

    def test_custom_response(self):
        # GIVEN
        response_body = "<Response>Ok</Response>"
        config = Config(response=response_body)
        hookee_manager = HookeeManager(config=config)
        hookee_manager._init_server_and_tunnel()

        webhook_url = "{}/webhook".format(hookee_manager.tunnel.public_url)

        # WHEN
        response = requests.get(webhook_url)

        # THEN
        self.assertEqual(response.content.decode("utf-8"), response_body)

        # WHEN
        hookee_manager.stop()

        # Wait for things to tear down
        time.sleep(2)



from hookee import HookeeManager
from hookee.pluginmanager import PluginManager

from hookee import util
from tests.testcase import HookeeTestCase

__author__ = "Alex Laird"
__copyright__ = "Copyright 2020, Alex Laird"
__version__ = "1.2.0"


class TestUtil(HookeeTestCase):
    def setUp(self):
        super(TestUtil, self).setUp()

        self.hookee_manager = HookeeManager()
        self.plugin_manager = PluginManager(self.hookee_manager)
        self.plugin = self.plugin_manager.get_plugin("request_url_info")

    def test_get_functions(self):
        # WHEN
        funcs = util.get_functions(self.plugin.module)

        # THEN
        self.assertEqual(funcs, ["run", "setup"])

    def get_args(self):
        # WHEN
        args = util.get_args(self.plugin.module.run)

        # THEN
        self.assertEqual(args, ["request"])



import os
from unittest import mock

from hookee.cli import hookee
from tests.testcase import HookeeTestCase

__author__ = "Alex Laird"
__copyright__ = "Copyright 2020, Alex Laird"
__version__ = "2.0.0"


class TestCli(HookeeTestCase):
    def test_update_config(self):
        # WHEN
        result = self.runner.invoke(hookee, ["update-config", "port", "1000"])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertIn("\"port\" has been updated", result.output)

    def test_update_invalid_config(self):
        # WHEN
        result = self.runner.invoke(hookee, ["update-config", "foo", "bar"])

        # THEN
        self.assertEqual(result.exit_code, 2)
        self.assertIn("No such key", result.output)

    def test_available_plugins(self):
        # WHEN
        result = self.runner.invoke(hookee, ["available-plugins"])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertIn(" * blueprint_default", result.output)
        self.assertIn(" * custom_plugin", result.output)
        self.assertIn(" * request_body", result.output)
        self.assertIn(" * request_files", result.output)
        self.assertIn(" * request_headers", result.output)
        self.assertIn(" * request_query_params", result.output)
        self.assertIn(" * request_url_info", result.output)
        self.assertIn(" * response_echo", result.output)
        self.assertIn(" * response_info", result.output)

    def test_enabled_plugins(self):
        # GIVEN
        self.config.remove("plugins", "request_files")
        self.config.remove("plugins", "request_query_params")

        # WHEN
        result = self.runner.invoke(hookee, ["enabled-plugins"])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertIn(" * blueprint_default", result.output)
        self.assertIn(" * request_url_info", result.output)
        self.assertIn(" * request_headers", result.output)
        self.assertIn(" * request_body", result.output)
        self.assertIn(" * response_echo", result.output)
        self.assertIn(" * response_info", result.output)

    def test_enable_plugin(self):
        # WHEN
        result = self.runner.invoke(hookee, ["enable-plugin", "request_body"])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertIn("\"request_body\" has been enabled", result.output)

    def test_enable_plugin_invalid(self):
        # WHEN
        result = self.runner.invoke(hookee, ["enable-plugin", "fake_plugin"])

        # THEN
        self.assertEqual(result.exit_code, 2)
        self.assertIn("\"fake_plugin\" could not be found", result.output)

    def test_disable_plugin(self):
        # WHEN
        result = self.runner.invoke(hookee, ["disable-plugin", "request_body"])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertIn("\"request_body\" is disabled", result.output)

    def test_disable_required_plugin(self):
        # WHEN
        result = self.runner.invoke(hookee, ["disable-plugin", "blueprint_default"])

        # THEN
        self.assertEqual(result.exit_code, 2)
        self.assertIn("Can't disable", result.output)

    @mock.patch("hookee.hookeemanager.HookeeManager.run")
    def test_start(self, mock_hookee_run):
        # WHEN
        result = self.runner.invoke(hookee, ["start"])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertTrue(mock_hookee_run.called)

    @mock.patch("hookee.hookeemanager.HookeeManager.run")
    def test_no_command_calls_start(self, mock_hookee_run):
        # WHEN
        result = self.runner.invoke(hookee)

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertTrue(mock_hookee_run.called)

    @mock.patch("hookee.hookeemanager.HookeeManager.run")
    def test_start_with_script_import(self, mock_hookee_run):
        # GIVEN
        builtin_plugin_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "..", "hookee", "plugins",
                                           "request_body.py")

        # WHEN
        result = self.runner.invoke(hookee, ["--response-script", builtin_plugin_path])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertTrue(mock_hookee_run.called)

    def test_start_with_invalid_script_import(self):
        # WHEN
        result = self.runner.invoke(hookee, ["--response-script", "no_such_file.py"])

        # THEN
        self.assertEqual(result.exit_code, 2)
        self.assertIn("'no_such_file.py' does not exist", result.output)

    @mock.patch("confuse.Configuration.set_args")
    @mock.patch("hookee.hookeemanager.HookeeManager.run")
    def test_start_arg_passed_to_config(self, mock_hookee_run, mock_set_args):
        # GIVEn
        response = "\"<Response>Ok</Response>\""

        # WHEN
        result = self.runner.invoke(hookee, ["--response", response])

        # THEN
        self.assertEqual(result.exit_code, 0)
        self.assertTrue(mock_hookee_run.called)
        self.assertTrue(mock_set_args.called)
        call_args, call_kwargs = mock_set_args.call_args
        self.assertIn("response", call_args[0])
        self.assertEqual(call_args[0]["response"], response)



import requests

from hookee import util
from tests.managedtestcase import ManagedTestCase

__author__ = "Alex Laird"
__copyright__ = "Copyright 2020, Alex Laird"
__version__ = "2.0.0"


class TestHookeeManager(ManagedTestCase):
    def test_http_get_query_params(self):
        # GIVEN
        params = {"param_1": ["param_value_1"]}

        # WHEN
        with self.captured_output() as (out, err):
            response = requests.get(self.webhook_url, params=params)

        self.assertEqual(response.status_code, 200)
        if util.python36_gte():
            self.assertIn("""Query Params: {
    "param_1": "param_value_1"
}""", out.getvalue())
        else:
            self.assertIn("""Query Params: {
    "param_1": [
        "param_value_1"
    ]
}""", out.getvalue())

    def test_http_post_form_data(self):
        # GIVEN
        data = {"form_data_1": "form_data_value_1"}

        # WHEN
        with self.captured_output() as (out, err):
            response = requests.post(self.webhook_url, data=data)

        self.assertEqual(response.status_code, 200)
        self.assertIn("\"Content-Type\": \"application/x-www-form-urlencoded\"", out.getvalue())
        self.assertEqual(response.headers.get("Content-Type"), "application/json")
        if util.python36_gte():
            self.assertIn("""Body: {
    "form_data_1": "form_data_value_1"
}""", out.getvalue())
            self.assertEqual(response.json(), data)
        else:
            self.assertIn("""Body: {
    "form_data_1": [
        "form_data_value_1"
    ]
}""", out.getvalue())
            self.assertEqual(response.json(), {"form_data_1": ["form_data_value_1"]})

    def test_http_post_json_data(self):
        # GIVEN
        headers = {"Content-Type": "application/json"}
        data = {"json_data_1": "json_data_value_1"}

        # WHEN
        with self.captured_output() as (out, err):
            response = requests.post(self.webhook_url, headers=headers, json=data)

        self.assertEqual(response.status_code, 200)
        self.assertIn("\"Content-Type\": \"application/json\"", out.getvalue())
        self.assertIn("""Body: {
    "json_data_1": "json_data_value_1"
}""", out.getvalue())
        self.assertEqual(response.headers.get("Content-Type"), "application/json")
        self.assertEqual(response.json(), data)



import os
from types import ModuleType

from flask import Response
from hookee.exception import HookeePluginValidationError

from hookee.pluginmanager import PluginManager, Plugin, VALID_PLUGIN_TYPES, BLUEPRINT_PLUGIN

from hookee import HookeeManager
from tests.testcase import HookeeTestCase

__author__ = "Alex Laird"
__copyright__ = "Copyright 2020, Alex Laird"
__version__ = "1.2.4"


class TestPluginManager(HookeeTestCase):
    def setUp(self):
        super(TestPluginManager, self).setUp()

        self.hookee_manager = HookeeManager()
        self.plugin_manager = PluginManager(self.hookee_manager)

    def test_build_from_module(self):
        # GIVEN
        plugin = self.plugin_manager.source.load_plugin("request_body")

        # WHEN
        plugin = Plugin.build_from_module(plugin)

        # THEN
        self.assertTrue(isinstance(plugin, Plugin))
        self.assertTrue(isinstance(plugin.module, ModuleType))
        self.assertEqual(plugin.plugin_type, "request")
        self.assertEqual(plugin.name, "request_body")
        self.assertTrue(plugin.has_setup)

    def test_build_from_module_no_plugin_type(self):
        # GIVEN
        plugin = self.plugin_manager.source.load_plugin("no_plugin_type")

        # WHEN
        with self.assertRaises(HookeePluginValidationError) as cm:
            Plugin.build_from_module(plugin)

        # THEN
        self.assertIn("does not conform to the plugin spec", str(cm.exception))

    def test_build_from_module_no_run(self):
        # GIVEN
        plugin = self.plugin_manager.source.load_plugin("no_run")

        # WHEN
        with self.assertRaises(HookeePluginValidationError) as cm:
            Plugin.build_from_module(plugin)

        # THEN
        self.assertIn("must implement `run", str(cm.exception))

    def test_build_from_module_wrong_args(self):
        # GIVEN
        plugin = self.plugin_manager.source.load_plugin("wrong_args")

        # WHEN
        with self.assertRaises(HookeePluginValidationError) as cm:
            Plugin.build_from_module(plugin)

        # THEN
        self.assertIn("`run(request)` must be defined", str(cm.exception))

    def test_build_from_module_no_blueprint(self):
        # GIVEN
        plugin = self.plugin_manager.source.load_plugin("no_blueprint")

        # WHEN
        with self.assertRaises(HookeePluginValidationError) as cm:
            Plugin.build_from_module(plugin)

        # THEN
        self.assertIn("must define `blueprint", str(cm.exception))

    def test_response_callback(self):
        # GIVEN
        def response_callback(request, response):
            response.data = "<Response>Ok</Response>"
            response.headers["Content-Type"] = "application/xml"
            return response

        self.assertEqual(0, len(self.plugin_manager.loaded_plugins))
        self.hookee_manager.config.response_callback = response_callback

        # WHEN
        self.plugin_manager.load_plugins()

        # THEN
        self.assertIsNotNone(self.plugin_manager.response_callback)
        response = self.plugin_manager.response_callback(None, Response())
        self.assertEqual(response.data.decode("utf-8"), "<Response>Ok</Response>")
        self.assertEqual(response.headers["Content-Type"], "application/xml")

    def test_load_plugins(self):
        # GIVEN
        self.assertEqual(0, len(self.plugin_manager.loaded_plugins))
        custom_plugin_path = os.path.join(self.plugins_dir, "custom_plugin.py")
        custom_request_plugin_path = os.path.join(self.plugins_dir, "custom_request_plugin.py")
        custom_response_plugin_path = os.path.join(self.plugins_dir, "custom_response_plugin.py")

        self.hookee_manager.config.append("plugins", "custom_plugin")
        self.hookee_manager.config.set("request_script", custom_request_plugin_path)
        self.hookee_manager.config.set("response_script", custom_response_plugin_path)
        self.hookee_manager.config.set("response", "<Response>Ok</Response>")
        self.hookee_manager.config.set("content_type", "application/xml")

        # WHEN
        self.plugin_manager.load_plugins()

        # THEN
        self.assertEqual(11, len(self.plugin_manager.loaded_plugins))
        request_script_found = False
        response_script_found = False
        for plugin in self.plugin_manager.loaded_plugins:
            self.assertTrue(isinstance(plugin, Plugin))
            self.assertIn(plugin.plugin_type, VALID_PLUGIN_TYPES)
            self.assertTrue(hasattr(plugin.module, "plugin_type"))
            self.assertIn(plugin.module.plugin_type, VALID_PLUGIN_TYPES)
            self.assertTrue(hasattr(plugin.module, "setup"))
            if plugin.plugin_type != BLUEPRINT_PLUGIN:
                self.assertTrue(hasattr(plugin.module, "run"))

            if plugin.name == "custom_request_plugin":
                request_script_found = True
            elif plugin.name == "custom_response_plugin":
                response_script_found = True
        self.assertTrue(request_script_found)
        self.assertTrue(response_script_found)
        # The last plugin should always be the response_script that we added
        self.assertEqual(self.plugin_manager.loaded_plugins[-1].name, "custom_response_plugin")
        self.assertIsNotNone(self.plugin_manager.response_callback)
        response = self.plugin_manager.response_callback(None, Response())
        self.assertEqual(response.data.decode("utf-8"), "<Response>Ok</Response>")
        self.assertEqual(response.headers["Content-Type"], "application/xml")
